use threads;
use threads::shared;
use strict;
use Test::More tests => 51;
use DE_EPAGES::Core::API::Script qw (RunScript);
use DE_EPAGES::WebInterface::API::Config qw (GetRequestRouterMonitors);
use DE_EPAGES::WebInterface::API::MessageCenter;
use DE_EPAGES::WebInterface::API::RequestRouterMonitor;

RunScript( Sub => sub {

    my $aMC = GetRequestRouterMonitors()->[0];
    die "No MONITOR port found in WebInterface.conf" unless $aMC;

    my ($ServerName, $MonitorPort) = @$aMC;
    my $Monitor = DE_EPAGES::WebInterface::API::RequestRouterMonitor->new(
        Address => $ServerName,
        Port => $MonitorPort
    );
    ok( $Monitor, "connected to MONITOR port" );

    my $aOriginalServers = $Monitor->servers;
    my $OriginalCountServers = scalar @$aOriginalServers;

    my $AppServerAddress = '0.0.0.0';
    my $AppServerPort = 12345;
    my $MessageCenter = DE_EPAGES::WebInterface::API::MessageCenter->new(
       AppServerAddress => $AppServerAddress,
       AppServerPort    => $AppServerPort,
    );
    $MessageCenter->connect;
    ok( $MessageCenter, "connected to MESSAGECNETER port" );

    ok( $OriginalCountServers == scalar @{$Monitor->servers}, "not yet added to server list" );

    $MessageCenter->sendInitialFreeRequest;
    my $PoolName = $MessageCenter->{'AppServerPoolName'};

    ok( 1+ $OriginalCountServers == scalar @{$Monitor->servers}, "now added to server list" );
    my $hServer = GetServerInfo($Monitor, $AppServerAddress, $AppServerPort);
    is( $hServer->{'ServerIP'}, $AppServerAddress, 'Server.ServerIP' );
    is( $hServer->{'ServerPort'}, $AppServerPort, 'Server.ServerPort' );
    is( $hServer->{'StoreName'}, '(NULL)', 'Server.StoreName' );
    is( $hServer->{'Status'}, 'idle', 'Server.Status = idle' );
    is( $hServer->{'Pool'}, $PoolName, 'Server.Pool' );
    is( $hServer->{'PID'}, $$, 'Server.PID' );
    is( $hServer->{'SiteGUID'}, '(NULL)', 'Server.SiteGUID' );
    is( $hServer->{'Status'}, 'idle', 'Server.Status' );
    is( $hServer->{'Extension'}, '(NULL)', 'Server.Extension' );
    is( $hServer->{'ClientIP'}, '(NULL)', 'Server.ClientIP' );
    is( $hServer->{'URL'}, '(NULL)', 'Server.URL' );
    is( $hServer->{'LongRequests'}, '0', 'Server.LongRequests' );
    is( $hServer->{'QualityHits'}, '0', 'Server.QualityHits' );
    is( $hServer->{'Hits'}, '0', 'Server.Hits' );
    is( $hServer->{'Mean'}, '0', 'Server.Mean' );
    is( $hServer->{'CacheItemCount'}, '0', 'Server.CacheItemCount' );
    is( $hServer->{'Overflow'}, '0', 'Server.Overflow' );
    cmp_ok( $hServer->{'LastContact'}, '<=', '1', 'Server.LastContact' );
    is( $hServer->{'Priority'}, '0', 'Server.Priority' );

    $MessageCenter->sendWorkingRequest;

    $hServer = GetServerInfo($Monitor, $AppServerAddress, $AppServerPort);
    ok( $hServer->{'Status'} eq 'working', 'Server.Status = working' );

    $MessageCenter->sendMonitorRequest;

    $hServer = GetServerInfo($Monitor, $AppServerAddress, $AppServerPort);
    ok( $hServer->{'Status'} eq 'monitor', 'Server.Status = monitor' );

    my $hPool = GetPoolInfo($Monitor, $PoolName);
    ok( $hPool->{'Name'} eq $PoolName, "Pool.Name" );
    ok( $hPool->{'CountServer'} =~ /\d+/, "Pool.CountServer" );
    ok( $hPool->{'ByNewHost'} =~ /\d+/, "Pool.ByNewHost" );
    ok( $hPool->{'CountMC'} =~ /\d+/, "Pool.CountMC" );
    ok( $hPool->{'Hits'} =~ /\d+/, "Pool.Hits" );
    ok( $hPool->{'Misses'} =~ /\d+/, "Pool.Misses" );
    ok( $hPool->{'Timeouts'} =~ /\d+/, "Pool.Timeouts" );
    ok( $hPool->{'CountServerCache'} =~ /\d+/, "Pool.CountServerCache" );
    ok( $hPool->{'CountMCCache'} == 0, "Pool.CountMCCache" );
    ok( $hPool->{'WorkingServer'} == 0, "Pool.WorkingServer" );
    ok( $hPool->{'AssignedServer'} == 0, "Pool.AssignedServer" );
    ok( $hPool->{'MonitorServer'} == 1, "Pool.MonitorServer" );
    ok( $hPool->{'IdleServer'} =~ /\d+/, "Pool.IdleServer" );
    ok( $hPool->{'ShutdownServer'} == 0, "Pool.ShutdownServer" );
    ok( $hPool->{'ByClientIP'} =~ /\d+/, "Pool.ByClientIP" );
    ok( $hPool->{'ByPerformance'} =~ /\d+/, "Pool.ByPerformance" );
    ok( $hPool->{'BySiteGuid'} =~ /\d+/, "Pool.BySiteGuid" );
    ok( $hPool->{'LimitByClient'} =~ /\d+/, "Pool.LimitByClient" );
    ok( $hPool->{'LimitByExtension'} =~ /\d+/, "Pool.LimitByExtension" );
    ok( $hPool->{'LimitBySite'} =~ /\d+/, "Pool.LimitBySite" );
    ok( $hPool->{'LimitBySpecialSite'} =~ /\d+/, "Pool.LimitBySpecialSite" );
    # obsolte
    ok( $hPool->{'ByIdleHost'} == 0, "Pool.ByIdleHost" );
    ok( $hPool->{'ByFreeHost'} == 0, "Pool.ByFreeHost" );
    ok( $hPool->{'BySingleHost'} == 0, "Pool.BySingleHost" );

    $MessageCenter->sendIdleRequest;

    $hServer = GetServerInfo($Monitor, $AppServerAddress, $AppServerPort);
    ok( $hServer->{'Status'} eq 'idle', 'Server.Status = idle' );

    $MessageCenter->sendFinishRequest;

    ok( $OriginalCountServers == scalar @{$Monitor->servers}, "removed from server list" );
});

sub GetServerInfo {
    my ( $Monitor, $AppServerAddress, $AppServerPort ) = @_;

    my @MatchingServers = grep {
            $_->{'ServerIP'} eq $AppServerAddress &&
            $_->{'ServerPort'} == $AppServerPort
        }
        @{$Monitor->servers};

    die "Server $AppServerAddress:$AppServerPort not found" unless scalar @MatchingServers;

    return $MatchingServers[0];
}

sub GetPoolInfo {
    my ( $Monitor, $PoolName ) = @_;

    my @MatchingPools = grep { $_->{'Name'} eq $PoolName } @{$Monitor->pools};
    die "Pool $PoolName not found" unless scalar @MatchingPools;

    return $MatchingPools[0];
}
